/*!	@file
	
	@brief GREP support library
	
	@author wakura
	@date 2008/04/28
*/
/*
	Copyright (C) 2008, wakura

	This software is provided 'as-is', without any express or implied
	warranty. In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose, 
	including commercial applications, and to alter it and redistribute it 
	freely, subject to the following restrictions:

		1. The origin of this software must not be misrepresented;
		   you must not claim that you wrote the original software.
		   If you use this software in a product, an acknowledgment
		   in the product documentation would be appreciated but is
		   not required.

		2. Altered source versions must be plainly marked as such, 
		   and must not be misrepresented as being the original software.

		3. This notice may not be removed or altered from any source
		   distribution.
*/
#pragma once

#include "stdafx.h"
#include <vector>
#include <windows.h>
#include <string.h>
#include <tchar.h>
#include "CGrepEnumKeys.h"

typedef std::pair< LPTSTR, DWORD > PairGrepEnumItem;
typedef std::vector< PairGrepEnumItem > VPGrepEnumItem;

class CGrepEnumFileBase {
private:
	VPGrepEnumItem m_vpItems;

public:
	CGrepEnumFileBase(){
	}

	virtual ~CGrepEnumFileBase(){
		ClearItems();
	}

	void ClearItems( void ){
		for( int i = 0; i < GetCount(); i++ ){
			LPTSTR lp = m_vpItems[ i ].first;
			m_vpItems[ i ].first = NULL;
			delete [] lp;
		}
		m_vpItems.clear();
		return;
	}

	BOOL IsExist( LPCTSTR lpFileName ){
		for( int i = 0; i < GetCount(); i++ ){
			if( _tcscmp( m_vpItems[ i ].first, lpFileName ) == 0 ){
				return TRUE;
			}
		}
		return FALSE;
	}

	virtual BOOL IsValid( WIN32_FIND_DATA& w32fd ){
		if( ! IsExist( w32fd.cFileName ) ){
			return TRUE;
		}
		return FALSE;
	}

	int GetCount( void ){
		return (int)m_vpItems.size();
	}

	LPCTSTR GetFileName( int i ){
		if( i < 0 || i >= GetCount() ) return NULL;
		return m_vpItems[ i ].first;
	}

	DWORD GetFileSizeLow( int i ){
		if( i < 0 || i >= GetCount() ) return NULL;
		return m_vpItems[ i ].second;
	}

	int Enumerates( LPCTSTR lpBaseFolder, VGrepEnumKeys& vecKeys ){
		int found = 0;

		for( int i = 0; i < (int)vecKeys.size(); i++ ){
			LPTSTR lpPath = new TCHAR[ _tcslen( lpBaseFolder ) + _tcslen( vecKeys[ i ] ) + 1 ];
			if( NULL == lpPath ) break;
			_stprintf( lpPath, _T("%s%s"), lpBaseFolder, vecKeys[ i ] );

			WIN32_FIND_DATA w32fd;
			HANDLE handle = ::FindFirstFile( lpPath, &w32fd );
			if( INVALID_HANDLE_VALUE != handle ){
				do{
					if( IsValid( w32fd ) ){
						LPTSTR lpName = new TCHAR[ _tcslen( w32fd.cFileName ) + 1 ];
						_tcscpy( lpName, w32fd.cFileName );
						m_vpItems.push_back( PairGrepEnumItem( lpName, w32fd.nFileSizeLow ) );
					}
				}while( ::FindNextFile( handle, &w32fd ) );
				::FindClose( handle );
			}
			delete [] lpPath;
		}
		return found;
	}
};
